@@ -1,18 +1,17 @@
 # This file is part of the MapProxy project.
 # Copyright (C) 2010 Omniscale <http://omniscale.de>
 # 
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU Affero General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
 # 
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU Affero General Public License for more details.
+#    http://www.apache.org/licenses/LICENSE-2.0
 # 
-# You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
 """
 Configuration loading and system initializing.
@@ -20,21 +19,23 @@ Configuration loading and system initializing.
 from __future__ import with_statement, division
 
 import os
+import sys
 import hashlib
 import urlparse
-import yaml #pylint: disable-msg=F0401
+import warnings
 from copy import deepcopy
 
 import logging
-log = logging.getLogger(__name__)
+log = logging.getLogger('mapproxy.config')
 
 from mapproxy.srs import SRS
 from mapproxy.util.ext.odict import odict
 from mapproxy.cache.file import FileCache, DummyCache
 from mapproxy.util.lock import SemLock
+from mapproxy.util.yaml import load_yaml_file, YAMLError
 from mapproxy.config.config import load_default_config
 from mapproxy.client.http import auth_data_from_url, HTTPClient
-
+from mapproxy.image.opts import ImageOptions, compatible_image_options, ImageFormat
 
 def loader(loaders, name):
     """
@@ -76,9 +77,11 @@ from mapproxy.layer import (
 from mapproxy.client.tile import TileClient, TileURLTemplate
 from mapproxy.client.wms import WMSClient, WMSInfoClient, WMSLegendClient, WMSLegendURLClient
 from mapproxy.service.wms import WMSServer, WMSLayer, WMSGroupLayer
+from mapproxy.service.wmts import WMTSServer
 from mapproxy.service.tile import TileServer, TileLayer
 from mapproxy.service.kml import KMLServer
 from mapproxy.service.demo import DemoServer
+from mapproxy.service.ows import OWSServer
 from mapproxy.source import DebugSource, DummySource
 from mapproxy.source.wms import WMSSource, WMSInfoSource, WMSLegendSource
 from mapproxy.source.tile import TiledSource
@@ -86,6 +89,7 @@ from mapproxy.cache.tile import TileManager
 from mapproxy.cache.legend import LegendCache
 from mapproxy.util import local_base_config, memoize
 from mapproxy.config.coverage import load_coverage
+from mapproxy.config.spec import validate_mapproxy_conf
 from mapproxy.featureinfo import XSLTransformer, has_xslt_support
 
 class ConfigurationError(Exception):
@@ -115,10 +119,11 @@ class ProxyConfiguration(object):
     def load_grids(self):
         self.grids = {}
         
-        self.grids['GLOBAL_GEODETIC'] = GridConfiguration(dict(srs='EPSG:4326'), context=self)
-        self.grids['GLOBAL_MERCATOR'] = GridConfiguration(dict(srs='EPSG:900913'), context=self)
+        self.grids['GLOBAL_GEODETIC'] = GridConfiguration(dict(srs='EPSG:4326', name='GLOBAL_GEODETIC'), context=self)
+        self.grids['GLOBAL_MERCATOR'] = GridConfiguration(dict(srs='EPSG:900913', name='GLOBAL_MERCATOR'), context=self)
         
         for grid_name, grid_conf in self.configuration.get('grids', {}).iteritems():
+            grid_conf.setdefault('name', grid_name)
             self.grids[grid_name] = GridConfiguration(grid_conf, context=self)
     
     def load_caches(self):
@@ -132,7 +137,7 @@ class ProxyConfiguration(object):
             self.caches[cache_name] = CacheConfiguration(conf=cache_conf, context=self)
     
     def load_sources(self):
-        self.sources = {}
+        self.sources = SourcesCollection()
         for source_name, source_conf in self.configuration.get('sources', {}).iteritems():
             self.sources[source_name] = SourceConfiguration.load(conf=source_conf, context=self)
     
@@ -276,8 +281,6 @@ class ConfigurationBase(object):
     """
     Base class for all configurations.
     """
-    optional_keys = set()
-    required_keys = set()
     defaults = {}
     
     def __init__(self, conf, context):
@@ -286,30 +289,13 @@ class ConfigurationBase(object):
         :param context: the complete proxy configuration
         :type context: ProxyConfiguration
         """
-        self.conf = {}
+        self.conf = conf
         self.context = context
-        expected_keys = set(self.optional_keys)
-        expected_keys.update(self.required_keys)
-        expected_keys.update(self.defaults.keys())
-        for k, v in conf.iteritems():
-            if k not in expected_keys:
-                log.warn('unexpected key %s', k)
-            self.conf[k] = v
-        
-        for k in self.required_keys:
-            if k not in self.conf:
-                raise ConfigurationError('missing key %s' % k)
-        
         for k, v in self.defaults.iteritems():
             if k not in self.conf:
                 self.conf[k] = v
 
 class GridConfiguration(ConfigurationBase):
-    optional_keys = set('''res srs bbox bbox_srs num_levels tile_size base
-        stretch_factor max_shrink_factor align_resolutions_with min_res max_res
-        res_factor threshold_res
-        '''.split())
-    
     @memoize
     def tile_grid(self):
         if 'base' in self.conf:
@@ -337,6 +323,7 @@ class GridConfiguration(ConfigurationBase):
         
         
         grid = tile_grid(
+            name=conf['name'],
             srs=conf.get('srs'),
             tile_size=tile_size,
             min_res=conf.get('min_res'),
@@ -349,31 +336,32 @@ class GridConfiguration(ConfigurationBase):
             num_levels=conf.get('num_levels'),
             stretch_factor=stretch_factor,
             max_shrink_factor=max_shrink_factor,
-            align_with = align_with,
+            align_with=align_with,
+            origin=conf.get('origin')
         )
         
         return grid
 
 
 class GlobalConfiguration(ConfigurationBase):
-    optional_keys = set('image grid srs http cache'.split())
-    
     def __init__(self, conf_base_dir, conf, context):
         ConfigurationBase.__init__(self, conf, context)
         self.base_config = load_default_config()
         self._copy_conf_values(self.conf, self.base_config)
         self.base_config.conf_base_dir = conf_base_dir
         mapproxy.config.finish_base_config(self.base_config)
+        
+        self.image_options = ImageOptionsConfiguration(self.conf.get('image', {}), context)
     
     def _copy_conf_values(self, d, target):
         for k, v in d.iteritems():
             if v is None: continue
-            if hasattr(v, 'iteritems'):
+            if hasattr(v, 'iteritems') and k in target:
                 self._copy_conf_values(v, target[k])
             else:
                 target[k] = v
     
-    def get_value(self, key, local, global_key=None, default_key=None):
+    def get_value(self, key, local={}, global_key=None, default_key=None):
         result = dotted_dict_get(key, local)
         if result is None:
             result = dotted_dict_get(global_key or key, self.conf)
@@ -394,6 +382,82 @@ class GlobalConfiguration(ConfigurationBase):
         
     
 
+default_image_options = {
+}
+
+class ImageOptionsConfiguration(ConfigurationBase):
+    def __init__(self, conf, context):
+        ConfigurationBase.__init__(self, conf, context)
+        self._init_formats()
+    
+    def _init_formats(self):
+        self.formats = {}
+        
+        formats_config = default_image_options.copy()
+        for format, conf in self.conf.get('formats', {}).iteritems():
+            if format in formats_config:
+                tmp = formats_config[format].copy()
+                tmp.update(conf)
+                conf = tmp
+            if 'resampling_method' in conf:
+                conf['resampling'] = conf.pop('resampling_method')
+            if 'encoding_options' in conf:
+                self._check_encoding_options(conf['encoding_options'])
+            formats_config[format] = conf
+        for format, conf in formats_config.iteritems():
+            if 'format' not in conf and format.startswith('image/'):
+                conf['format'] = format
+            self.formats[format] = conf
+    
+    def _check_encoding_options(self, options):
+        if not options:
+            return
+        options = options.copy()
+        jpeg_quality = options.pop('jpeg_quality', None)
+        if jpeg_quality and not isinstance(jpeg_quality, int):
+            raise ConfigurationError('jpeg_quality is not an integer')
+        quantizer = options.pop('quantizer', None)
+        if quantizer and quantizer not in ('fastoctree', 'mediancut'):
+            raise ConfigurationError('unknown quantizer')
+        
+        if options:
+            raise ConfigurationError('unknown encoding_options: %r' % options)
+    
+    def image_opts(self, image_conf, format):
+        conf = {}
+        if format in self.formats:
+            conf = self.formats[format].copy()
+        
+        resampling = image_conf.get('resampling_method') or conf.get('resampling')
+        if resampling is None:
+            resampling = self.context.globals.get_value('image.resampling_method', {})
+        transparent = image_conf.get('transparent')
+        opacity = image_conf.get('opacity')
+        img_format = image_conf.get('format')
+        colors = image_conf.get('colors')
+        mode = image_conf.get('mode')
+        encoding_options = image_conf.get('encoding_options')
+        
+        self._check_encoding_options(encoding_options)
+        
+        # only overwrite default if it is not None
+        for k, v in dict(transparent=transparent, opacity=opacity, resampling=resampling,
+            format=img_format, colors=colors, mode=mode, encoding_options=encoding_options).iteritems():
+            if v is not None:
+                conf[k] = v
+        
+        if 'format' not in conf and format and format.startswith('image/'):
+            conf['format'] = format
+        
+        # force 256 colors for image.paletted for backwards compat
+        paletted = self.context.globals.get_value('image.paletted', self.conf)
+        if conf.get('colors') is None and 'png' in conf.get('format', '') and paletted:
+            conf['colors'] = 256
+
+        opts = ImageOptions(**conf)
+        return opts
+    
+    
 def dotted_dict_get(key, d):
     """
     >>> dotted_dict_get('foo', {'foo': {'bar': 1}})
@@ -412,21 +476,72 @@ def dotted_dict_get(key, d):
         return None
     return d
 
+
+class SourcesCollection(dict):
+    """
+    Collection of SourceConfigurations.
+    Allows access to tagged WMS sources, e.g. 
+    ``sc['source_name:lyr,lyr2']`` will return the source with ``source_name``
+    and set ``req.layers`` to ``lyr1,lyr2``.
+    """
+    def __getitem__(self, key):
+        layers = None
+        source_name = key
+        if ':' in source_name:
+            source_name, layers = source_name.split(':', 1)
+        source = dict.__getitem__(self, source_name)
+        if not layers:
+            return source
+
+        if source.conf.get('type') not in ('wms', 'mapserver'):
+            raise ConfigurationError("found ':' in non-WMS source name: '%s'."
+                " tagged sources only supported for WMS" % key)
+
+        source = deepcopy(source)
+        
+        supported_layers = source.conf['req'].get('layers', [])
+        supported_layer_set = SourcesCollection.layer_set(supported_layers)
+        layer_set = SourcesCollection.layer_set(layers)
+
+        if supported_layer_set and not layer_set.issubset(supported_layer_set):
+            raise ConfigurationError('layers (%s) not supported by source (%s)' % (
+                layers, supported_layers))
+
+        source.conf['req']['layers'] = layers
+        return source
+
+    def __contains__(self, key):
+        source_name = key
+        if ':' in source_name:
+            source_name, _ = source_name.split(':', 1)
+        return dict.__contains__(self, source_name)
+
+    @staticmethod
+    def layer_set(layers):
+        if isinstance(layers, (list, tuple)):
+            return set(layers)
+        return set(layers.split(','))
+
+
 class SourceConfiguration(ConfigurationBase):
     @classmethod
     def load(cls, conf, context):
         source_type = conf['type']
-        for subclass in cls.__subclasses__():
-            if source_type in subclass.source_type:
-                return subclass(conf, context)
         
-        raise ValueError("unknown source type '%s'" % source_type)
+        subclass = source_configuration_types.get(source_type)
+        if not subclass:
+            raise ConfigurationError("unknown source type '%s'" % source_type)
+
+        return subclass(conf, context)
     
     @memoize
     def coverage(self):
         if not 'coverage' in self.conf: return None
         return load_coverage(self.conf['coverage'])
     
+    def image_opts(self, format=None):
+        return self.context.globals.image_options.image_opts(self.conf.get('image', {}), format)
+    
     def http_client(self, url):
         http_client = None
         url, (username, password) = auth_data_from_url(url)
@@ -454,10 +569,6 @@ def resolution_range(conf):
 
 class WMSSourceConfiguration(SourceConfiguration):
     source_type = ('wms',)
-    optional_keys = set('''type supported_srs supported_formats image
-        wms_opts http concurrent_requests coverage seed_only
-        min_res max_res min_scale max_scale'''.split())
-    required_keys = set('req'.split())
     
     @staticmethod
     def static_legend_source(url, context):
@@ -480,6 +591,14 @@ class WMSSourceConfiguration(SourceConfiguration):
             fi_transformer = XSLTransformer(fi_xslt)
         return fi_transformer
     
+    def image_opts(self, format=None):
+        if 'transparent' not in self.conf.get('image', {}):
+            transparent = self.conf['req'].get('transparent')
+            if transparent is not None:
+                transparent = bool(str(transparent).lower() == 'true')
+                self.conf.setdefault('image', {})['transparent'] = transparent
+        return SourceConfiguration.image_opts(self, format=format)
+    
     def source(self, params=None):
         if not self.conf.get('wms_opts', {}).get('map', True):
             return None
@@ -493,13 +612,8 @@ class WMSSourceConfiguration(SourceConfiguration):
         if request_format:
             params['format'] = request_format
         
-        transparent = self.conf['req'].get('transparent', 'false')
-        transparent = bool(str(transparent).lower() == 'true')
-        
-        opacity = self.conf.get('image', {}).get('opacity')
-        
-        resampling = self.context.globals.get_value('image.resampling_method', self.conf)
-        
+        image_opts = self.image_opts(format=params.get('format'))
+
         supported_srs = [SRS(code) for code in self.conf.get('supported_srs', [])]
         supported_formats = [file_ext(f) for f in self.conf.get('supported_formats', [])]
         version = self.conf.get('wms_opts', {}).get('version', '1.1.1')
@@ -525,15 +639,16 @@ class WMSSourceConfiguration(SourceConfiguration):
         
         http_method = self.context.globals.get_value('http.method', self.conf)
         
-        request = create_request(self.conf['req'], params, version=version)
+        request = create_request(self.conf['req'], params, version=version,
+            abspath=self.context.globals.abspath)
         http_client, request.url = self.http_client(request.url)
-        client = WMSClient(request, supported_srs, http_client=http_client, 
-                           http_method=http_method, resampling=resampling, lock=lock,
-                           supported_formats=supported_formats or None)
-        return WMSSource(client, transparent=transparent, coverage=coverage,
-                         res_range=res_range, opacity=opacity,
-                         transparent_color=transparent_color,
-                         transparent_color_tolerance=transparent_color_tolerance)
+        client = WMSClient(request, http_client=http_client, 
+                           http_method=http_method, lock=lock)
+        return WMSSource(client, image_opts=image_opts, coverage=coverage,
+                         res_range=res_range, transparent_color=transparent_color,
+                         transparent_color_tolerance=transparent_color_tolerance,
+                         supported_srs=supported_srs,
+                         supported_formats=supported_formats or None)
     
     def fi_source(self, params=None):
         if params is None: params = {}
@@ -548,7 +663,8 @@ class WMSSourceConfiguration(SourceConfiguration):
             if 'featureinfo_format' in wms_opts:
                 params['info_format'] = wms_opts['featureinfo_format']
             fi_request = create_request(self.conf['req'], params,
-                req_type='featureinfo', version=version)
+                req_type='featureinfo', version=version,
+                abspath=self.context.globals.abspath)
             
             fi_transformer = self.fi_xslt_transformer(self.conf.get('wms_opts', {}),
                                                      self.context)
@@ -580,7 +696,8 @@ class WMSSourceConfiguration(SourceConfiguration):
             for lg_layer in lg_layers:
                 lg_req['layer'] = lg_layer
                 lg_request = create_request(lg_req, params,
-                    req_type='legendgraphic', version=version)
+                    req_type='legendgraphic', version=version,
+                    abspath=self.context.globals.abspath)
                 http_client, lg_request.url = self.http_client(lg_request.url)
                 lg_client = WMSLegendClient(lg_request, http_client=http_client)
                 lg_clients.append(lg_client)
@@ -589,11 +706,59 @@ class WMSSourceConfiguration(SourceConfiguration):
         return lg_source
     
 
+class MapServerSourceConfiguration(WMSSourceConfiguration):
+    source_type = ('mapserver',)
+    
+    def __init__(self, conf, context):
+        WMSSourceConfiguration.__init__(self, conf, context)
+        self.script = self.context.globals.get_path('mapserver.binary',
+            self.conf)
+        if not self.script or not os.path.isfile(self.script):
+            raise ConfigurationError('could not find mapserver binary (%r)' %
+                (self.script, ))
+        
+        # set url to dummy script name, required as identifier
+        # for concurrent_request
+        self.conf.setdefault('req', {})['url'] = 'http://localhost' + self.script
+    
+    def http_client(self, url):
+        working_dir = self.context.globals.get_path('mapserver.working_dir', self.conf)
+        if working_dir and not os.path.isdir(working_dir):
+            raise ConfigurationError('could not find mapserver working_dir (%r)' % (working_dir, ))
+        
+        from mapproxy.client.cgi import CGIClient
+        client = CGIClient(script=self.script, working_directory=working_dir)
+        return client, url
+
+
+class MapnikSourceConfiguration(SourceConfiguration):
+    source_type = ('mapnik',)
+    
+    def source(self, params=None):
+        if not self.context.seed and self.conf.get('seed_only'):
+            return DummySource()
+        
+        image_opts = self.image_opts()
+        
+        lock = None
+        concurrent_requests = self.context.globals.get_value('concurrent_requests', self.conf,
+                                                        global_key='http.concurrent_requests')
+        if concurrent_requests:
+            lock_dir = self.context.globals.get_path('cache.lock_dir', self.conf)
+            md5 = hashlib.md5(self.conf['mapfile'])
+            lock_file = os.path.join(lock_dir, md5.hexdigest() + '.lck')
+            lock = lambda: SemLock(lock_file, concurrent_requests)
+        
+        coverage = self.coverage()
+        res_range = resolution_range(self.conf)
+        
+        mapfile = self.context.globals.abspath(self.conf['mapfile'])
+        from mapproxy.source.mapnik import MapnikSource
+        return MapnikSource(mapfile, image_opts=image_opts, coverage=coverage,
+                         res_range=res_range, lock=lock)
+
 class TileSourceConfiguration(SourceConfiguration):
     source_type = ('tile',)
-    optional_keys = set('''type grid request_format origin coverage seed_only
-                           transparent http'''.split())
-    required_keys = set('url'.split())
     defaults = {'origin': 'sw', 'grid': 'GLOBAL_MERCATOR'}
     
     def source(self, params=None):
@@ -612,14 +777,14 @@ class TileSourceConfiguration(SourceConfiguration):
         http_client, url = self.http_client(url)
         grid = self.context.grids[self.conf['grid']].tile_grid()
         coverage = self.coverage()
-        opacity = self.conf.get('image', {}).get('opacity')
-        transparent = self.conf.get('transparent')
+        image_opts = self.image_opts()
         
         inverse = True if origin == 'nw' else False
         format = file_ext(params['format'])
-        client = TileClient(TileURLTemplate(url, format=format), http_client=http_client)
-        return TiledSource(grid, client, inverse=inverse, coverage=coverage, opacity=opacity,
-                           transparent=transparent)
+        client = TileClient(TileURLTemplate(url, format=format), http_client=http_client, grid=grid)
+        return TiledSource(grid, client, inverse=inverse, coverage=coverage,
+            image_opts=image_opts)
+
 
 def file_ext(mimetype):
     _mime_class, format, _options = split_mime_type(mimetype)
@@ -632,19 +797,28 @@ class DebugSourceConfiguration(SourceConfiguration):
     def source(self, params=None):
         return DebugSource()
 
+
+source_configuration_types = {
+    'wms': WMSSourceConfiguration,
+    'tile': TileSourceConfiguration,
+    'debug': DebugSourceConfiguration,
+    'mapserver': MapServerSourceConfiguration,
+    'mapnik': MapnikSourceConfiguration,
+}
+
+
 class CacheConfiguration(ConfigurationBase):
-    optional_keys = set('''format cache_dir grids link_single_color_images image
-        use_direct_from_res use_direct_from_level meta_buffer meta_size
-        minimize_meta_requests disable_storage'''.split())
-    optional_keys.update(tile_filter_conf_keys)
-    required_keys = set('name sources'.split())
     defaults = {'format': 'image/png', 'grids': ['GLOBAL_MERCATOR']}
     
     def cache_dir(self):
+        if ('cache_dir' not in self.conf 
+            and 'base_dir' not in self.context.globals.conf.get('cache', {})):
+            warnings.warn('globals.cache.base_dir not defined. default value '
+            '(../var/cache_data) will be changed with 1.2.0.', FutureWarning)
         return self.context.globals.get_path('cache_dir', self.conf,
             global_key='cache.base_dir')
         
-    def _file_cache(self, grid_conf):
+    def _file_cache(self, grid_conf, file_ext):
         if self.conf.get('disable_storage', False):
             return DummyCache()
         
@@ -653,11 +827,14 @@ class CacheConfiguration(ConfigurationBase):
         suffix = grid_conf.conf['srs'].replace(':', '')
         cache_dir = os.path.join(cache_dir, self.conf['name'] + '_' + suffix)
         link_single_color_images = self.conf.get('link_single_color_images', False)
+        if link_single_color_images and sys.platform == 'win32':
+            log.warn('link_single_color_images not supported on windows')
+            link_single_color_images = False
         
-        tile_filter = self._tile_filter()
-        return FileCache(cache_dir, file_ext=file_ext(self.conf['format']),
-            pre_store_filter=tile_filter,
-            link_single_color_images=link_single_color_images)
+        lock_timeout = self.context.globals.get_value('http.client_timeout', {})
+        
+        return FileCache(cache_dir, file_ext=file_ext,
+            lock_timeout=lock_timeout, link_single_color_images=link_single_color_images)
     
     def _tile_filter(self):
         filters = []
@@ -667,9 +844,23 @@ class CacheConfiguration(ConfigurationBase):
                 filters.append(f)
         return filters
     
+    @memoize
+    def image_opts(self):
+        format = None
+        if 'format' not in self.conf.get('image', {}):
+            format = self.conf.get('format') or self.conf.get('request_format')
+        image_opts = self.context.globals.image_options.image_opts(self.conf.get('image', {}), format)
+        if image_opts.format is None:
+            if format is not None and format.startswith('image/'):
+                image_opts.format = ImageFormat(format)
+            else:
+                image_opts.format = ImageFormat('image/png')
+        return image_opts
+        
     @memoize
     def caches(self):
-        request_format = self.conf.get('request_format') or self.conf['format']
+        base_image_opts = self.image_opts()
+        request_format = self.conf.get('request_format') or self.conf.get('format')
         caches = []
 
         meta_buffer = self.context.globals.get_value('meta_buffer', self.conf,
@@ -678,19 +869,31 @@ class CacheConfiguration(ConfigurationBase):
             global_key='cache.meta_size')
         minimize_meta_requests = self.context.globals.get_value('minimize_meta_requests', self.conf,
             global_key='cache.minimize_meta_requests')
+        concurrent_tile_creators = self.context.globals.get_value('concurrent_tile_creators', self.conf,
+            global_key='cache.concurrent_tile_creators')
         
         for grid_conf in [self.context.grids[g] for g in self.conf['grids']]:
             sources = []
-            for source_conf in [self.context.sources[s] for s in self.conf['sources']]:
+            source_image_opts = []
+            for source_name in self.conf['sources']:
+                if not source_name in self.context.sources:
+                    raise ConfigurationError('unknown source %s' % source_name)
+                source_conf = self.context.sources[source_name]
                 source = source_conf.source({'format': request_format})
                 if source:
                     sources.append(source)
+                    source_image_opts.append(source.image_opts)
             assert sources, 'no sources configured for %s' % self.conf['name']
-            cache = self._file_cache(grid_conf)
             tile_grid = grid_conf.tile_grid()
-            mgr = TileManager(tile_grid, cache, sources, file_ext(request_format),
+            tile_filter = self._tile_filter()
+            image_opts = compatible_image_options(source_image_opts, base_opts=base_image_opts)
+            cache = self._file_cache(grid_conf, image_opts.format.ext)
+            mgr = TileManager(tile_grid, cache, sources, image_opts.format.ext,
+                              image_opts=image_opts,
                               meta_size=meta_size, meta_buffer=meta_buffer,
-                              minimize_meta_requests=minimize_meta_requests)
+                              minimize_meta_requests=minimize_meta_requests,
+                              concurrent_tile_creators=concurrent_tile_creators,
+                              pre_store_filter=tile_filter)
             extent = merge_layer_extents(sources)
             if extent.is_default:
                 extent = map_extent_from_grid(tile_grid)
@@ -699,21 +902,22 @@ class CacheConfiguration(ConfigurationBase):
     
     @memoize
     def map_layer(self):
-        resampling = self.context.globals.get_value('image.resampling_method', self.conf)
-        opacity = self.conf.get('image', {}).get('opacity')
+        image_opts = self.image_opts()
+        max_tile_limit = self.context.globals.get_value('max_tile_limit', self.conf,
+            global_key='cache.max_tile_limit')
         caches = []
         main_grid = None
         for grid, extent, tile_manager in self.caches():
             if main_grid is None:
                 main_grid = grid
-            caches.append((CacheMapLayer(tile_manager, extent=extent, resampling=resampling,
-                                         opacity=opacity),
+            caches.append((CacheMapLayer(tile_manager, extent=extent, image_opts=image_opts,
+                                         max_tile_limit=max_tile_limit),
                           (grid.srs,)))
         
         if len(caches) == 1:
             layer = caches[0][0]
         else:
-            layer = SRSConditional(caches, caches[0][0].extent, caches[0][0].transparent, opacity=opacity)
+            layer = SRSConditional(caches, caches[0][0].extent, caches[0][0].transparent, opacity=image_opts.opacity)
         
         if 'use_direct_from_level' in self.conf:
             self.conf['use_direct_from_res'] = main_grid.resolution(self.conf['use_direct_from_level'])
@@ -722,15 +926,11 @@ class CacheConfiguration(ConfigurationBase):
                 raise ValueError('use_direct_from_level/res only supports single sources')
             source_conf = self.context.sources[self.conf['sources'][0]]
             layer = ResolutionConditional(layer, source_conf.source(), self.conf['use_direct_from_res'],
-                                          main_grid.srs, layer.extent, opacity=opacity)
+                                          main_grid.srs, layer.extent, opacity=image_opts.opacity)
         return layer
 
 
 class WMSLayerConfiguration(ConfigurationBase):
-    optional_keys = set('name layers min_res max_res min_scale '
-                        'max_scale legendurl sources'.split())
-    required_keys = set('title'.split())
-    
     @memoize
     def wms_layer(self):
         layers = []
@@ -755,9 +955,6 @@ class WMSLayerConfiguration(ConfigurationBase):
         return layer
 
 class LayerConfiguration(ConfigurationBase):
-    optional_keys = set('min_res max_res min_scale max_scale legendurl sources layers'.split())
-    required_keys = set('name title'.split())
-
     @memoize
     def wms_layer(self):
         sources = []
@@ -818,7 +1015,7 @@ class LayerConfiguration(ConfigurationBase):
                 md['name'] = self.conf['name']
                 md['name_path'] = (self.conf['name'], grid.srs.srs_code.replace(':', '').upper())
                 md['name_internal'] = md['name_path'][0] + '_' + md['name_path'][1]
-                md['format'] = self.context.caches[cache_name].conf['format']
+                md['format'] = self.context.caches[cache_name].image_opts().format
             
                 tile_layers.append(TileLayer(self.conf['name'], self.conf['title'],
                                              md, cache_source))
@@ -838,15 +1035,20 @@ def fi_xslt_transformers(conf, context):
     return fi_transformers
 
 class ServiceConfiguration(ConfigurationBase):
-    optional_keys = set('wms tms kml demo'.split())
-    
     def services(self):
-        services = {}
+        services = []
+        ows_services = []
         for service_name, service_conf in self.conf.iteritems():
             creator = getattr(self, service_name + '_service', None)
             if not creator:
                 raise ValueError('unknown service: %s' % service_name)
-            services[service_name] = creator(service_conf or {})
+            if service_name in ('wms', 'wmts'):
+                ows_services.append(creator(service_conf or {}))
+            else:
+                services.append(creator(service_conf or {}))
+        
+        if ows_services:
+            services.append(OWSServer(ows_services))
         return services
     
     def tile_layers(self, conf):
@@ -860,14 +1062,24 @@ class ServiceConfiguration(ConfigurationBase):
     def kml_service(self, conf):
         md = self.context.services.conf.get('wms', {}).get('md', {}).copy()
         md.update(conf.get('md', {}))
+        max_tile_age = self.context.globals.get_value('tiles.expires_hours')
+        max_tile_age *= 60 * 60 # seconds
         layers = self.tile_layers(conf)
-        return KMLServer(layers, md)
+        return KMLServer(layers, md, max_tile_age=max_tile_age)
     
     def tms_service(self, conf):
         md = self.context.services.conf.get('wms', {}).get('md', {}).copy()
         md.update(conf.get('md', {}))
+        max_tile_age = self.context.globals.get_value('tiles.expires_hours')
+        max_tile_age *= 60 * 60 # seconds
         layers = self.tile_layers(conf)
-        return TileServer(layers, md)
+        return TileServer(layers, md, max_tile_age=max_tile_age)
+    
+    def wmts_service(self, conf):
+        md = self.context.services.conf.get('wms', {}).get('md', {}).copy()
+        md.update(conf.get('md', {}))
+        layers = self.tile_layers(conf)
+        return WMTSServer(layers, md)
     
     def wms_service(self, conf):
         md = conf.get('md', {})
@@ -883,8 +1095,19 @@ class ServiceConfiguration(ConfigurationBase):
         concurrent_layer_renderer = self.context.globals.get_value(
             'concurrent_layer_renderer', conf,
             global_key='wms.concurrent_layer_renderer')
-        image_formats = self.context.globals.get_value('image_formats', conf,
+        image_formats_names = self.context.globals.get_value('image_formats', conf,
                                                        global_key='wms.image_formats')
+        image_formats = {}
+        for format in image_formats_names:
+            if format.startswith('image/'):
+                opts = ImageOptions(format=format)
+            else:
+                # lookup custom format
+                opts = self.context.globals.image_options.image_opts({}, format)
+            if opts.format in image_formats:
+                log.warn('duplicate mime-type for WMS image_formats: "%s" already configured',
+                    opts.format)
+            image_formats[opts.format] = opts
         info_types = conf.get('featureinfo_types')
         srs = self.context.globals.get_value('srs', conf, global_key='wms.srs')
         self.context.globals.base_config.wms.srs = srs
@@ -905,34 +1128,30 @@ class ServiceConfiguration(ConfigurationBase):
             layers[layer_name] = layer_conf.wms_layer()
         tile_layers = self.tile_layers(conf)
         image_formats = self.context.globals.get_value('image_formats', conf, global_key='wms.image_formats')
-        return DemoServer(layers, md, tile_layers=tile_layers, image_formats=image_formats)
+        srs = self.context.globals.get_value('srs', conf, global_key='wms.srs')
+        return DemoServer(layers, md, tile_layers=tile_layers,
+            image_formats=image_formats, srs=srs)
     
-def load_services(conf_file):
-    conf = load_configuration(conf_file)
-    return conf.configured_services()
-
-def load_yaml_file(file):
-    if yaml.__with_libyaml__:
-        return yaml.load(file, Loader=yaml.CLoader)
-    else:
-        return yaml.load(file)
 
 def load_configuration(mapproxy_conf, seed=False):
-    if hasattr(mapproxy_conf, 'read'):
-        conf_data = mapproxy_conf.read()
-        conf_base_dir = os.getcwd()
-    else:
-        log.info('Reading services configuration: %s' % mapproxy_conf)
-        conf_data = open(mapproxy_conf).read()
-        conf_base_dir = os.path.abspath(os.path.dirname(mapproxy_conf))
-    conf_dict = load_yaml_file(conf_data)
-    if 'base' in conf_dict:
-        with open(os.path.join(conf_base_dir, conf_dict['base'])) as f:
-            base_dict = load_yaml_file(f)
-        if 'base' in base_dict:
-            log.warn('found `base` option in base config but recursive inheritance is not supported.')
-        conf_dict = merge_dict(conf_dict, base_dict)
+    log.info('reading: %s' % mapproxy_conf)
+    conf_base_dir = os.path.abspath(os.path.dirname(mapproxy_conf))
     
+    try:
+        conf_dict = load_yaml_file(mapproxy_conf)
+        if 'base' in conf_dict:
+            base_conf_file = conf_dict.pop('base')
+            base_dict = load_yaml_file(os.path.join(conf_base_dir, base_conf_file))
+            if 'base' in base_dict:
+                log.warn('found `base` option in base config but recursive inheritance is not supported.')
+            conf_dict = merge_dict(conf_dict, base_dict)
+    except YAMLError, ex:
+        raise ConfigurationError(ex)
+    errors, informal_only = validate_mapproxy_conf(conf_dict)
+    for error in errors:
+        log.warn(error)
+    if not informal_only:
+        raise ConfigurationError('invalid configuration')
     return ProxyConfiguration(conf_dict, conf_base_dir=conf_base_dir, seed=seed)
 
 def merge_dict(conf, base):
@@ -972,4 +1191,4 @@ def parse_color(color):
     
     return r, g, b
     
-    
\ No newline at end of file
+    
