@@ -1,18 +1,17 @@
 # This file is part of the MapProxy project.
 # Copyright (C) 2010 Omniscale <http://omniscale.de>
 # 
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU Affero General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
 # 
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU Affero General Public License for more details.
+#    http://www.apache.org/licenses/LICENSE-2.0
 # 
-# You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
 """
 (Meta-)Tile grids (data and calculations).
@@ -20,11 +19,8 @@
 from __future__ import division
 import math
 
-from mapproxy.config import base_config
-from mapproxy.srs import SRS, get_epsg_num
-
-import logging
-log = logging.getLogger(__name__)
+from mapproxy.srs import SRS, get_epsg_num, merge_bbox
+from mapproxy.util.collections import ImmutableDictList
 
 geodetic_epsg_codes = [4326]
 
@@ -94,8 +90,8 @@ default_bboxs = _default_bboxs()
 def tile_grid(srs=None, bbox=None, bbox_srs=None, tile_size=(256, 256),
               res=None, res_factor=2.0, threshold_res=None,
               num_levels=None, min_res=None, max_res=None,
-              stretch_factor=None, max_shrink_factor=None,
-              align_with=None
+              stretch_factor=1.15, max_shrink_factor=4.0,
+              align_with=None, origin='ll', name=None
               ):
     """
     This function creates a new TileGrid.
@@ -112,7 +108,11 @@ def tile_grid(srs=None, bbox=None, bbox_srs=None, tile_size=(256, 256),
     
     if res:
         if isinstance(res, list):
-            res = sorted(res, reverse=True)
+            if isinstance(res[0], (tuple, list)):
+                # named resolutions
+                res = sorted(res, key=lambda x: x[1], reverse=True)
+            else:
+                res = sorted(res, reverse=True)
             assert min_res is None
             assert max_res is None
             assert align_with is None
@@ -126,14 +126,15 @@ def tile_grid(srs=None, bbox=None, bbox_srs=None, tile_size=(256, 256),
         res = resolutions(min_res, max_res, res_factor, num_levels, bbox, tile_size)
     
     return TileGrid(srs, bbox=bbox, tile_size=tile_size, res=res, threshold_res=threshold_res,
-                    stretch_factor=stretch_factor, max_shrink_factor=max_shrink_factor)
+                    stretch_factor=stretch_factor, max_shrink_factor=max_shrink_factor,
+                    origin=origin, name=name)
 
 def aligned_resolutions(min_res=None, max_res=None, res_factor=2.0, num_levels=None,
                 bbox=None, tile_size=(256, 256), align_with=None):
     
     
     alinged_res = align_with.resolutions
-    res = alinged_res[:]
+    res = list(alinged_res)
     
     if not min_res:
         width = bbox[2] - bbox[0]
@@ -222,7 +223,19 @@ def bbox_height(bbox):
 
 def bbox_size(bbox):
     return bbox_width(bbox), bbox_height(bbox)
-    
+
+
+class NamedGridList(ImmutableDictList):
+    def __init__(self, items):
+        tmp = []
+        for i, value in enumerate(items):
+            if isinstance(value, (tuple, list)):
+                name, value = value
+            else:
+                name = str('%02d' % i)
+            tmp.append((name, value))
+        ImmutableDictList.__init__(self, tmp)
+
 class TileGrid(object):
     """
     This class represents a regular tile grid. The first level (0) contains a single
@@ -237,10 +250,12 @@ class TileGrid(object):
     """
     
     spheroid_a = 6378137.0 # for 900913
+    flipped_y_axis = False
     
     def __init__(self, srs=900913, bbox=None, tile_size=(256, 256), res=None,
                  threshold_res=None, is_geodetic=False, levels=None,
-                 stretch_factor=None, max_shrink_factor=None):
+                 stretch_factor=1.15, max_shrink_factor=4.0, origin='ll',
+                 name=None):
         """
         :param stretch_factor: allow images to be scaled up by this factor
             before the next level will be selected
@@ -255,17 +270,17 @@ class TileGrid(object):
             srs = SRS(srs)
         self.srs = srs
         self.tile_size = tile_size
+        self.origin = origin
+        self.name = name
+
+        if origin in ('ul', 'nw'):
+            self.flipped_y_axis = True
+
         self.is_geodetic = is_geodetic
         
-        if stretch_factor is None:
-            stretch_factor = base_config().image.stretch_factor
         self.stretch_factor = stretch_factor
-        
-        if max_shrink_factor is None:
-            max_shrink_factor = base_config().image.max_shrink_factor
         self.max_shrink_factor = max_shrink_factor
         
-        
         if levels is None:
             self.levels = 20
         else:
@@ -275,17 +290,22 @@ class TileGrid(object):
             bbox = self._calc_bbox()
         self.bbox = bbox
         
+        factor = None
+        
         if res is None:
-            res = self._calc_res()
+            factor = 2.0
+            res = self._calc_res(factor=factor)
         elif res == 'sqrt2':
             if levels is None:
                 self.levels = 40
-            res = self._calc_res(factor=math.sqrt(2))
+            factor = math.sqrt(2)
+            res = self._calc_res(factor=factor)
         elif is_float(res):
-            res = self._calc_res(factor=float(res))
-
+            factor = float(res)
+            res = self._calc_res(factor=factor)
+        
         self.levels = len(res)
-        self.resolutions = res
+        self.resolutions = NamedGridList(res)
         self.threshold_res = threshold_res
         
         self.grid_sizes = self._calc_grids()
@@ -294,11 +314,11 @@ class TileGrid(object):
         width = self.bbox[2] - self.bbox[0]
         height = self.bbox[3] - self.bbox[1]
         grids = []
-        for res in self.resolutions:
-            x = math.ceil(width // res / self.tile_size[0])
-            y = math.ceil(height // res / self.tile_size[1])
-            grids.append((int(x), int(y)))
-        return grids
+        for idx, res in self.resolutions.iteritems():
+            x = max(math.ceil(width // res / self.tile_size[0]), 1)
+            y = max(math.ceil(height // res / self.tile_size[1]), 1)
+            grids.append((idx, (int(x), int(y))))
+        return NamedGridList(grids)
     
     def _calc_bbox(self):
         if self.is_geodetic:
@@ -316,8 +336,7 @@ class TileGrid(object):
             return pyramid_res_level(initial_res, levels=self.levels)
         else:
             return pyramid_res_level(initial_res, factor, levels=self.levels)
-    
-    
+
     def resolution(self, level):
         """
         Returns the resolution of the `level` in units/pixel.
@@ -387,7 +406,10 @@ class TileGrid(object):
         """
         res = self.resolution(level)
         x = x - self.bbox[0]
-        y = y - self.bbox[1]
+        if self.flipped_y_axis:
+            y = self.bbox[3] - y
+        else:
+            y = y - self.bbox[1]
         tile_x = x/float(res*self.tile_size[0])
         tile_y = y/float(res*self.tile_size[1])
         return (int(math.floor(tile_x)), int(math.floor(tile_y)), level)
@@ -405,7 +427,27 @@ class TileGrid(object):
         """
         return (x, self.grid_sizes[z][1]-1-y, z)
     
-    def get_affected_tiles(self, bbox, size, req_srs=None, inverse=False):
+    def supports_access_with_origin(self, origin):
+        if self.origin == origin:
+            return True
+        grid_size = self.grid_sizes[0]
+        level_0_bbox = self._tiles_bbox([(0, 0, 0), 
+            (grid_size[0] - 1, grid_size[1] - 1, 0)])
+        
+        if self.bbox[1] == level_0_bbox[1] and self.bbox[3] == level_0_bbox[3]:
+            return True
+        else:
+            return False
+    
+    def origin_tile(self, level, origin):
+        assert self.supports_access_with_origin(origin), 'TODO error exception'
+        tile = (0, 0, level)
+        if self.origin != origin:
+            tile = self.flip_tile_coord(tile)
+        
+        return tile
+        
+    def get_affected_tiles(self, bbox, size, req_srs=None):
         """
         Get a list with all affected tiles for a bbox and output size.
         
@@ -422,7 +464,7 @@ class TileGrid(object):
           <generator object ...>)
         """
         src_bbox, level = self.get_affected_bbox_and_level(bbox, size, req_srs=req_srs)
-        return self.get_affected_level_tiles(src_bbox, level, inverse=inverse)
+        return self.get_affected_level_tiles(src_bbox, level)
     
     def get_affected_bbox_and_level(self, bbox, size, req_srs=None):
         if req_srs and req_srs != self.srs:
@@ -441,7 +483,7 @@ class TileGrid(object):
         
         return src_bbox, level
     
-    def get_affected_level_tiles(self, bbox, level, inverse=False):
+    def get_affected_level_tiles(self, bbox, level):
         """
         Get a list with all affected tiles for a `bbox` in the given `level`.
         :returns: the bbox, the size and a list with tile coordinates, sorted row-wise
@@ -459,30 +501,27 @@ class TileGrid(object):
         delta = self.resolutions[level] / 10.0
         x0, y0, _ = self.tile(bbox[0]+delta, bbox[1]+delta, level)
         x1, y1, _ = self.tile(bbox[2]-delta, bbox[3]-delta, level)
-        
         try:
-            return self._tile_iter(x0, y0, x1, y1, level, inverse=inverse)
+            return self._tile_iter(x0, y0, x1, y1, level)
         except IndexError:
             raise GridError('Invalid BBOX')
         
-    def _tile_iter(self, x0, y0, x1, y1, level, inverse=False):
+    def _tile_iter(self, x0, y0, x1, y1, level):
         xs = range(x0, x1+1)
-        if inverse:
-            y0 = int(self.grid_sizes[level][1]) - 1 - y0
-            y1 = int(self.grid_sizes[level][1]) - 1 - y1
-            ys = range(y1, y0+1)
+        if self.flipped_y_axis:
+            y0, y1 = y1, y0
+            ys = range(y0, y1+1)
         else:
             ys = range(y1, y0-1, -1)
+        
         ll = (xs[0], ys[-1], level)
         ur = (xs[-1], ys[0], level)
-        if inverse:
-            ll = self.flip_tile_coord(ll)
-            ur = self.flip_tile_coord(ur)
-        abbox = self._get_bbox([ll, ur])
+
+        abbox = self._tiles_bbox([ll, ur])
         return (abbox, (len(xs), len(ys)),
                 _create_tile_list(xs, ys, level, self.grid_sizes[level]))
         
-    def _get_bbox(self, tiles):
+    def _tiles_bbox(self, tiles):
         """
         Returns the bbox of multiple tiles.
         The tiles should be ordered row-wise, bottom-up.
@@ -490,33 +529,11 @@ class TileGrid(object):
         :param tiles: ordered list of tiles
         :returns: the bbox of all tiles
         """
-        ll = tiles[0]
-        ur = tiles[-1]
-        x0, y0 = self._get_south_west_point(ll)
-        x1, y1 = self._get_south_west_point((ur[0]+1, ur[1]+1, ur[2]))
-        return x0, y0, x1, y1
+        ll_bbox = self.tile_bbox(tiles[0])
+        ur_bbox = self.tile_bbox(tiles[-1])
+        return merge_bbox(ll_bbox, ur_bbox)
     
-    def _get_south_west_point(self, tile_coord):
-        """
-        Returns the coordinate of the lower left corner.
-        
-        :param tile_coord: the tile coordinate
-        :type tile_coord: ``(x, y, z)``
-        
-        >>> grid = TileGrid(SRS(900913))
-        >>> [round(x, 2) for x in grid._get_south_west_point((0, 0, 0))]
-        [-20037508.34, -20037508.34]
-        >>> [round(x, 2) for x in grid._get_south_west_point((1, 1, 1))]
-        [0.0, 0.0]
-        """
-        x, y, z = tile_coord
-        res = self.resolution(z)
-        
-        x0 = self.bbox[0] + round(x * res * self.tile_size[0], 12)
-        y0 = self.bbox[1] + round(y * res * self.tile_size[1], 12)
-        return x0, y0
-    
-    def tile_bbox(self, (x, y, z), limit=False):
+    def tile_bbox(self, tile_coord, limit=False):
         """
         Returns the bbox of the given tile.
         
@@ -526,18 +543,28 @@ class TileGrid(object):
         >>> [round(x, 2) for x in grid.tile_bbox((1, 1, 1))]
         [0.0, 0.0, 20037508.34, 20037508.34]
         """
-        x0, y0 = self._get_south_west_point((x, y, z))
+        x, y, z = tile_coord
         res = self.resolution(z)
-        width = round(res * self.tile_size[0], 12)
-        height = round(res * self.tile_size[1], 12)
+        
+        x0 = self.bbox[0] + round(x * res * self.tile_size[0], 12)
+        x1 = x0 + round(res * self.tile_size[0], 12)
+        
+        if self.flipped_y_axis:
+            y1 = self.bbox[3] - round(y * res * self.tile_size[1], 12)
+            y0 = y1 - round(res * self.tile_size[1], 12)
+        else:
+            y0 = self.bbox[1] + round(y * res * self.tile_size[1], 12)
+            y1 = y0 + round(res * self.tile_size[1], 12)
+        
         if limit:
             return (
                 max(x0, self.bbox[0]),
                 max(y0, self.bbox[1]),
-                min(x0+width, self.bbox[2]),
-                min(y0+height, self.bbox[3])
+                min(x1, self.bbox[2]),
+                min(y1, self.bbox[3])
             )
-        return (x0, y0, x0+width, y0+height)
+
+        return x0, y0, x1, y1
     
     def limit_tile(self, tile_coord):
         """
@@ -555,9 +582,12 @@ class TileGrid(object):
         (1, 2, 2)
         """
         x, y, z = tile_coord
-        grid = self.grid_sizes[z]
-        if z < 0 or z >= self.levels:
+        if isinstance(z, basestring):
+            if z not in self.grid_sizes:
+                return None
+        elif z < 0 or z >= self.levels:
             return None
+        grid = self.grid_sizes[z]
         if x < 0 or y < 0 or x >= grid[0] or y >= grid[1]:
             return None
         return x, y, z
@@ -638,7 +668,7 @@ class MetaGrid(object):
         if tiles:
             assert tile_coord is None
             level = tiles[0][2]
-            bbox = self.grid._get_bbox(tiles)
+            bbox = self.grid._tiles_bbox(tiles)
         else:
             level = tile_coord[2]
             bbox = self.unbuffered_meta_bbox(tile_coord)
@@ -649,18 +679,9 @@ class MetaGrid(object):
         x, y, z = tile_coord
         
         meta_size = self._meta_size(z)
-        
-        meta_x = x//meta_size[0]
-        meta_y = y//meta_size[1]
-        
-        (minx, miny, maxx, maxy) = self.grid.tile_bbox((meta_x * meta_size[0],
-                                                        meta_y * meta_size[1], z))
-        width = (maxx - minx) * meta_size[0]
-        height = (maxy - miny) * meta_size[1]
-        maxx = minx + width
-        maxy = miny + height
-        
-        return (minx, miny, maxx, maxy)
+
+        return self.grid._tiles_bbox([(tile_coord), 
+            (x+meta_size[0]-1, y+meta_size[1]-1, z)])
     
     def _buffered_bbox(self, bbox, level, limit_to_grid_bbox=True):
         minx, miny, maxx, maxy = bbox
@@ -761,8 +782,11 @@ class MetaGrid(object):
         
         grid_size = 1+maxx-minx, 1+maxy-miny
         
-        ys = range(maxy, miny-1, -1)
-        xs = range(minx, maxx+1)
+        if self.grid.flipped_y_axis:
+            ys = xrange(miny, maxy+1)
+        else:
+            ys = xrange(maxy, miny-1, -1)
+        xs = xrange(minx, maxx+1)
         
         bounds = (minx, miny, z), (maxx, maxy, z)
         
@@ -791,8 +815,11 @@ class MetaGrid(object):
         minx, miny, z = self.main_tile(main_tile)
         maxx = minx + tile_grid[0] - 1
         maxy = miny + tile_grid[1] - 1
-        ys = range(maxy, miny-1, -1)
-        xs = range(minx, maxx+1)
+        if self.grid.flipped_y_axis:
+            ys = xrange(miny, maxy+1)
+        else:
+            ys = xrange(maxy, miny-1, -1)
+        xs = xrange(minx, maxx+1)
         
         return list(_create_tile_list(xs, ys, z, self.grid.grid_sizes[z]))
         
@@ -818,7 +845,7 @@ class MetaGrid(object):
         """
         if tile:
             tiles = self._meta_tile_list(tile, grid_size)
-        
+
         for i in range(grid_size[1]):
             for j in range(grid_size[0]):
                 yield tiles[j+i*grid_size[0]], (
@@ -862,24 +889,16 @@ class MetaGrid(object):
         except IndexError:
             raise GridError('Invalid BBOX')
         
-    def _tile_iter(self, x0, y0, x1, y1, level, inverse=False):
+    def _tile_iter(self, x0, y0, x1, y1, level):
         meta_size = self._meta_size(level)
         
         xs = range(x0, x1+1, meta_size[0])
-        if inverse:
-            y0 = int(self.grid.grid_sizes[level][1]) - 1 - y0
-            y1 = int(self.grid.grid_sizes[level][1]) - 1 - y1
-            ys = range(y1, y0+1, meta_size[1])
-        else:
-            ys = range(y1, y0-1, -1 * meta_size[1])
+        ys = range(y1, y0-1, -1 * meta_size[1])
         ll = (xs[0], ys[-1], level)
         ur = (xs[-1], ys[0], level)
-        if inverse:
-            ll = self.grid.flip_tile_coord(ll)
-            ur = self.grid.flip_tile_coord(ur)
         # add meta_size to get full affected bbox
         ur = ur[0]+meta_size[0]-1, ur[1]+meta_size[1]-1, ur[2]
-        abbox = self.grid._get_bbox([ll, ur])
+        abbox = self.grid._tiles_bbox([ll, ur])
         return (abbox, (len(xs), len(ys)),
                 _create_tile_list(xs, ys, level, self.grid.grid_sizes[level]))
 
