@@ -2,24 +2,21 @@
 # This file is part of the MapProxy project.
 # Copyright (C) 2010 Omniscale <http://omniscale.de>
 # 
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU Affero General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
 # 
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU Affero General Public License for more details.
+#    http://www.apache.org/licenses/LICENSE-2.0
 # 
-# You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
 from __future__ import with_statement
 import os
 
-from paste.deploy.converters import asbool
-
 from mapproxy.request import Request
 from mapproxy.response import Response
 from mapproxy.util.collections import LRU
@@ -29,6 +26,16 @@ from threading import Lock
 import logging
 log = logging.getLogger(__name__)
 
+def asbool(value):
+    """
+    >>> all([asbool(True), asbool('trUE'), asbool('ON'), asbool(1)])
+    True
+    >>> any([asbool(False), asbool('false'), asbool('foo'), asbool(None)])
+    False
+    """
+    value = str(value).lower()
+    return value in ('1', 'true', 'yes', 'on')
+
 def app_factory(global_options, config_dir, allow_listing=False, **local_options):
     """
     Create a new MultiMapProxy app.
@@ -158,7 +165,7 @@ class DirectoryConfLoader(ConfLoader):
         return False
     
     def _is_conf_file(self, fname):
-        if not os.path.isfile(os.path.join(self.base_dir, fname)):
+        if not os.path.isfile(fname):
             return False
         if self.suffix:
             return fname.lower().endswith(self.suffix)
@@ -184,7 +191,7 @@ class DirectoryConfLoader(ConfLoader):
     def available_apps(self):
         apps = []
         for f in os.listdir(self.base_dir):
-            if self._is_conf_file(f):
+            if self._is_conf_file(os.path.join(self.base_dir, f)):
                 app_name = self.app_name_from_filename(f)
                 apps.append(app_name)
         return apps
