@@ -1,18 +1,17 @@
 # This file is part of the MapProxy project.
 # Copyright (C) 2010, 2011 Omniscale <http://omniscale.de>
 # 
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU Affero General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
 # 
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU Affero General Public License for more details.
+#    http://www.apache.org/licenses/LICENSE-2.0
 # 
-# You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
 from __future__ import with_statement, division
 
@@ -82,7 +81,7 @@ def timestamp():
     return datetime.now().strftime('%H:%M:%S')
 
 def format_bbox(bbox):
-    return ('(%.5f, %.5f, %.5f, %.5f)') % tuple(bbox)
+    return ('%.5f, %.5f, %.5f, %.5f') % tuple(bbox)
 
 def status_symbol(i, total):
     """
@@ -120,12 +119,12 @@ def exp_backoff(func, args=(), kw={}, max_repeat=10, start_backoff_sec=2,
 def format_seed_task(task):
     info = []
     info.append('  %s:' % (task.md['name'], ))
-    info.append("    Seeding cache '%s' with grid '%s' (%s)" % (
+    info.append("    Seeding cache '%s' with grid '%s' in %s" % (
                  task.md['cache_name'], task.md['grid_name'], task.grid.srs.srs_code))
     if task.coverage:
-        info.append('    Limited to: %s' % (format_bbox(task.coverage.extent.llbbox), ))
+        info.append('    Limited to: %s (EPSG:4326)' % (format_bbox(task.coverage.extent.llbbox), ))
     else:
-        info.append('   Complete grid: %s' % (format_bbox(map_extent_from_grid(task.grid).llbbox), ))
+        info.append('   Complete grid: %s (EPSG:4326)' % (format_bbox(map_extent_from_grid(task.grid).llbbox), ))
     info.append('    Levels: %s' % (task.levels, ))
         
     if task.refresh_timestamp:
@@ -139,12 +138,12 @@ def format_seed_task(task):
 def format_cleanup_task(task):
     info = []
     info.append('  %s:' % (task.md['name'], ))
-    info.append("    Cleaning up cache '%s' with grid '%s' (%s)" % (
+    info.append("    Cleaning up cache '%s' with grid '%s' in %s" % (
                  task.md['cache_name'], task.md['grid_name'], task.grid.srs.srs_code))
     if task.coverage:
-        info.append('    Limited to: %s' % (format_bbox(task.coverage.extent.llbbox), ))
+        info.append('    Limited to: %s (EPSG:4326)' % (format_bbox(task.coverage.extent.llbbox), ))
     else:
-        info.append('    Complete grid: %s' % (format_bbox(map_extent_from_grid(task.grid).llbbox), ))
+        info.append('    Complete grid: %s (EPSG:4326)' % (format_bbox(map_extent_from_grid(task.grid).llbbox), ))
     info.append('    Levels: %s' % (task.levels, ))
         
     if task.remove_timestamp:
