@@ -2,18 +2,17 @@
 # This file is part of the MapProxy project.
 # Copyright (C) 2010 Omniscale <http://omniscale.de>
 # 
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU Affero General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
 # 
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU Affero General Public License for more details.
+#    http://www.apache.org/licenses/LICENSE-2.0
 # 
-# You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
 from __future__ import division
 
@@ -25,12 +24,12 @@ from mapproxy.exception import RequestError
 from mapproxy.service.base import Server
 from mapproxy.request.tile import tile_request
 from mapproxy.request.base import split_mime_type
-from mapproxy.config import base_config
 from mapproxy.layer import map_extent_from_grid
 from mapproxy.source import SourceError
 from mapproxy.srs import SRS
 from mapproxy.grid import default_bboxs
 from mapproxy.image import BlankImageSource
+from mapproxy.image.opts import ImageOptions
 from mapproxy.util.ext.odict import odict
 
 import logging
@@ -40,9 +39,6 @@ log = logging.getLogger(__name__)
 from mapproxy.template import template_loader, bunch
 get_template = template_loader(__file__, 'templates')
 
-import logging
-log = logging.getLogger(__name__)
-
 class TileServer(Server):
     """
     A Tile Server. Supports strict TMS and non-TMS requests. The difference is the
@@ -57,10 +53,11 @@ class TileServer(Server):
     template_file = 'tms_capabilities.xml'
     layer_template_file = 'tms_tilemap_capabilities.xml'
 
-    def __init__(self, layers, md):
+    def __init__(self, layers, md, max_tile_age=None):
         Server.__init__(self)
         self.layers = layers
         self.md = md
+        self.max_tile_age = max_tile_age
     
     def map(self, tile_request):
         """
@@ -71,7 +68,7 @@ class TileServer(Server):
         tile = layer.render(tile_request, use_profiles=tile_request.use_profiles)
         resp = Response(tile.as_buffer(), content_type='image/' + tile_request.format)
         resp.cache_headers(tile.timestamp, etag_data=(tile.timestamp, tile.size),
-                           max_age=base_config().tiles.expires_hours * 60 * 60)
+                           max_age=self.max_tile_age)
         resp.make_conditional(tile_request.http)
         return resp
     
@@ -93,7 +90,9 @@ class TileServer(Server):
     
     def authorize_tile_layer(self, layer_name, env):
         if 'mapproxy.authorize' in env:
-            result = env['mapproxy.authorize']('tms', [layer_name])
+            result = env['mapproxy.authorize']('tms', [layer_name], environ=env)
+            if result['authorized'] == 'unauthenticated':
+                raise RequestError('unauthorized', status=401)
             if result['authorized'] == 'full':
                 return
             if result['authorized'] == 'partial':
@@ -103,7 +102,9 @@ class TileServer(Server):
     
     def authorized_tile_layers(self, env):
         if 'mapproxy.authorize' in env:
-            result = env['mapproxy.authorize']('tms', [l for l in self.layers])
+            result = env['mapproxy.authorize']('tms', [l for l in self.layers], environ=env)
+            if result['authorized'] == 'unauthenticated':
+                raise RequestError('unauthorized', status=401)
             if result['authorized'] == 'full':
                 return self.layers
             if result['authorized'] == 'none':
@@ -180,28 +181,30 @@ class TileLayer(object):
         tile_coord = self.grid.internal_tile_coord(tile_request.tile, use_profiles)
         if tile_coord is None:
             raise RequestError('The requested tile is outside the bounding box'
-                               ' of the tile map.', request=tile_request)
+                               ' of the tile map.', request=tile_request,
+                               code='TileOutOfRange')
         if tile_request.origin == 'nw':
             tile_coord = self.grid.flip_tile_coord(tile_coord)
         return tile_coord
     
     def empty_response(self):
         if not self._empty_tile:
-            img = BlankImageSource(size=self.grid.tile_size, transparent=True)
-            self._empty_tile = img.as_buffer(self.format)
+            img = BlankImageSource(size=self.grid.tile_size,
+                image_opts=ImageOptions(format=self.format, transparent=True))
+            self._empty_tile = img.as_buffer()
         return ImageResponse(self._empty_tile, time.time())
     
     def render(self, tile_request, use_profiles=False):
         if tile_request.format != self.format:
             raise RequestError('invalid format (%s). this tile set only supports (%s)'
-                               % (tile_request.format, self.format), request=tile_request)
+                               % (tile_request.format, self.format), request=tile_request,
+                               code='InvalidParameterValue')
         tile_coord = self._internal_tile_coord(tile_request, use_profiles=use_profiles)
         try:
             tile = self.tile_manager.load_tile_coord(tile_coord, with_metadata=True)
             if tile.source is None: return self.empty_response()
             return TileResponse(tile)
         except SourceError, e:
-            log.error(e)
             raise RequestError(e.args[0], request=tile_request, internal=True)
 
 class ImageResponse(object):
