@@ -1,18 +1,17 @@
 # This file is part of the MapProxy project.
 # Copyright (C) 2010 Omniscale <http://omniscale.de>
 # 
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU Affero General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
 # 
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU Affero General Public License for more details.
+#    http://www.apache.org/licenses/LICENSE-2.0
 # 
-# You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
 from __future__ import with_statement, division
 import os
@@ -22,26 +21,38 @@ from webtest import TestApp
 from mapproxy.wsgiapp import make_wsgi_app 
 
 def module_setup(test_config, config_file, with_cache_data=False):
+    prepare_env(test_config, config_file, with_cache_data)
+    create_app(test_config)
+
+def prepare_env(test_config, config_file, with_cache_data=False):
     fixture_dir = os.path.join(os.path.dirname(__file__), 'fixture')
+    test_config['fixture_dir'] = fixture_dir
     fixture_layer_conf = os.path.join(fixture_dir, config_file)
     
     if 'base_dir' not in test_config:
-        test_config['base_dir'] = tempfile.mkdtemp()
+        test_config['tmp_dir'] = tempfile.mkdtemp()
+        test_config['base_dir'] = os.path.join(test_config['tmp_dir'], 'etc')
+        os.mkdir(test_config['base_dir'])
     test_config['config_file'] = os.path.join(test_config['base_dir'], config_file)
     shutil.copy(fixture_layer_conf, test_config['config_file'])
     if with_cache_data:
         shutil.copytree(os.path.join(fixture_dir, 'cache_data'),
                         os.path.join(test_config['base_dir'], 'cache_data'))
+    
+def create_app(test_config):
     app = make_wsgi_app(test_config['config_file'])
-    app.application.base_config.debug_mode = True
+    app.base_config.debug_mode = True
     test_config['app'] = TestApp(app, use_unicode=False)
 
 def module_teardown(test_config):
     shutil.rmtree(test_config['base_dir'])
+    if 'tmp_dir' in test_config:
+        shutil.rmtree(test_config['tmp_dir'])
+
     test_config.clear()
     
 def make_base_config(test_config):
-    return lambda: test_config['app'].app.application.base_config
+    return lambda: test_config['app'].app.base_config
 
 class SystemTest(object):
     def setup(self):
