@@ -1,25 +1,24 @@
 # This file is part of the MapProxy project.
 # Copyright (C) 2010 Omniscale <http://omniscale.de>
 # 
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU Affero General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
 # 
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU Affero General Public License for more details.
+#    http://www.apache.org/licenses/LICENSE-2.0
 # 
-# You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
 from __future__ import with_statement, division
 
 from cStringIO import StringIO
 from mapproxy.request.wms import WMS111MapRequest
 from mapproxy.platform.image import Image
-from mapproxy.test.image import is_png
+from mapproxy.test.image import is_png, is_jpeg
 from mapproxy.test.system import module_setup, module_teardown, SystemTest
 from nose.tools import eq_
 
@@ -31,7 +30,7 @@ def setup_module():
 def teardown_module():
     module_teardown(test_config)
 
-class TestCoverageWMS(SystemTest):
+class TestSeedOnlyWMS(SystemTest):
     config = test_config
     def setup(self):
         SystemTest.setup(self)
@@ -58,4 +57,26 @@ class TestCoverageWMS(SystemTest):
         assert is_png(data)
         img = Image.open(data)
         eq_(img.mode, 'RGBA')
-        eq_(img.getcolors(), [(200*200, (255, 255, 255, 0))])
\ No newline at end of file
+        eq_(img.getcolors(), [(200*200, (255, 255, 255, 0))])
+
+class TestSeedOnlyTMS(SystemTest):
+    config = test_config
+
+    def test_get_tile_cached(self):
+        resp = self.app.get('/tms/1.0.0/wms_cache/0/0/1.jpeg')
+        resp.content_type = 'image/jpeg'
+        data = StringIO(resp.body)
+        assert is_jpeg(data)
+        img = Image.open(data)
+        eq_(img.mode, 'RGB')
+        # cached image has more that 256 colors, getcolors -> None
+        eq_(img.getcolors(), None)
+
+    def test_get_tile_uncached(self):
+        resp = self.app.get('/tms/1.0.0/wms_cache/0/0/0.jpeg')
+        resp.content_type = 'image/jpeg'
+        data = StringIO(resp.body)
+        assert is_jpeg(data)
+        img = Image.open(data)
+        eq_(img.mode, 'RGB')
+        eq_(img.getcolors(), [(256*256, (255, 255, 255))])
\ No newline at end of file
