@@ -1,18 +1,17 @@
 # This file is part of the MapProxy project.
 # Copyright (C) 2010 Omniscale <http://omniscale.de>
 # 
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU Affero General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
 # 
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU Affero General Public License for more details.
+#    http://www.apache.org/licenses/LICENSE-2.0
 # 
-# You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
 from __future__ import with_statement, division
 import os
@@ -124,8 +123,8 @@ class TestWMS111(WMSTest):
             'http://localhost/service?')
         layer_names = set(xml.xpath('//Layer/Layer/Name/text()'))
         expected_names = set(['direct', 'wms_cache', 'wms_cache_100', 'wms_cache_130',
-                              'wms_merge', 'tms_cache', 'wms_cache_multi',
-                              'wms_cache_link_single', 'wms_cache_110'])
+            'wms_cache_transparent', 'wms_merge', 'tms_cache', 'wms_cache_multi',
+            'wms_cache_link_single', 'wms_cache_110'])
         eq_(layer_names, expected_names)
         assert validate_with_dtd(xml, dtd_name='wms/1.1.1/WMS_MS_Capabilities.dtd')
     
@@ -172,7 +171,6 @@ class TestWMS111(WMSTest):
         self.common_map_req.params['styles'] = 'default'
         resp = self.app.get(self.common_map_req)
         eq_(resp.content_type, 'image/png')
-        print resp.body
         data = StringIO(resp.body)
         assert is_png(data)
         assert Image.open(data).mode == 'RGB'
@@ -183,14 +181,55 @@ class TestWMS111(WMSTest):
         data = StringIO(resp.body)
         assert is_png(data)
         assert Image.open(data).mode == 'RGB'
-        
+
+    def test_get_map_png8_custom_format(self):
+        self.common_map_req.params['layers'] = 'wms_cache'
+        self.common_map_req.params['format'] = 'image/png; mode=8bit'
+        resp = self.app.get(self.common_map_req)
+        eq_(resp.headers['Content-type'], 'image/png; mode=8bit')
+        data = StringIO(resp.body)
+        assert is_png(data)
+        img = Image.open(data)
+        eq_(img.mode, 'P')
+
+    def test_get_map_png_transparent_non_transparent_data(self):
+        self.common_map_req.params['transparent'] = 'True'
+        resp = self.app.get(self.common_map_req)
+        eq_(resp.content_type, 'image/png')
+        data = StringIO(resp.body)
+        assert is_png(data)
+        img = Image.open(data)
+        eq_(img.mode, 'RGB')
+
     def test_get_map_png_transparent(self):
+        self.common_map_req.params['layers'] = 'wms_cache_transparent'
         self.common_map_req.params['transparent'] = 'True'
         resp = self.app.get(self.common_map_req)
         eq_(resp.content_type, 'image/png')
         data = StringIO(resp.body)
         assert is_png(data)
         assert Image.open(data).mode == 'RGBA'
+
+    def test_get_map_png_w_default_bgcolor(self):
+        self.common_map_req.params['layers'] = 'wms_cache_transparent'
+        resp = self.app.get(self.common_map_req)
+        eq_(resp.content_type, 'image/png')
+        data = StringIO(resp.body)
+        assert is_png(data)
+        img = Image.open(data)
+        eq_(img.mode, 'RGB')
+        eq_(img.getcolors()[0][1], (255, 255, 255))
+
+    def test_get_map_png_w_bgcolor(self):
+        self.common_map_req.params['layers'] = 'wms_cache_transparent'
+        self.common_map_req.params['bgcolor'] = '0xff00a0'
+        resp = self.app.get(self.common_map_req)
+        eq_(resp.content_type, 'image/png')
+        data = StringIO(resp.body)
+        assert is_png(data)
+        img = Image.open(data)
+        eq_(img.mode, 'RGB')
+        eq_(sorted(img.getcolors())[-1][1], (255, 0, 160))
     
     def test_get_map_jpeg(self):
         self.common_map_req.params['format'] = 'image/jpeg'
@@ -335,7 +374,6 @@ class TestWMS111(WMSTest):
                         {'body': 'info', 'headers': {'content-type': 'text/plain'}})
         with mock_httpd(('localhost', 42423), [expected_req]):
             resp = self.app.get(self.common_fi_req)
-            print resp.body
             eq_(resp.content_type, 'text/plain')
             eq_(resp.body, 'info')
 
@@ -360,7 +398,6 @@ class TestWMS111(WMSTest):
             self.common_fi_req.params['bbox'] = '3570269,5540889,3643458,5653553'
             self.common_fi_req.params['srs'] = 'EPSG:31467'
             resp = self.app.get(self.common_fi_req)
-            print resp.body
             eq_(resp.content_type, 'text/plain')
             eq_(resp.body, 'info')
 
@@ -405,9 +442,9 @@ class TestWMS111(WMSTest):
             eq_(resp.body, 'info')
     
     def test_get_featureinfo_missing_params_strict(self):
-        request_parser = self.app.app.application.handlers['service'].request_parser
+        request_parser = self.app.app.handlers['service'].services['wms'].request_parser
         try:
-            self.app.app.application.handlers['service'].request_parser = \
+            self.app.app.handlers['service'].services['wms'].request_parser = \
                 functools.partial(wms_request, strict=True)
         
             del self.common_fi_req.params['format']
@@ -417,13 +454,13 @@ class TestWMS111(WMSTest):
             assert 'missing parameters' in xml.xpath('//ServiceException/text()')[0]
             assert validate_with_dtd(xml, 'wms/1.1.1/exception_1_1_1.dtd')
         finally:
-            self.app.app.application.handlers['service'].request_parser = request_parser
+            self.app.app.handlers['service'].services['wms'].request_parser = request_parser
+            self.app.app.handlers['service'].request_parser = request_parser
     
     def test_get_featureinfo_not_queryable(self):
         self.common_fi_req.params['query_layers'] = 'tms_cache'
         self.common_fi_req.params['exceptions'] = 'application/vnd.ogc.se_xml'
         resp = self.app.get(self.common_fi_req)
-        print resp.body
         eq_(resp.content_type, 'application/vnd.ogc.se_xml')
         xml = resp.lxml
         eq_(xml.xpath('/ServiceExceptionReport/ServiceException/@code'), [])
@@ -463,8 +500,8 @@ class TestWMS110(WMSTest):
         
         layer_names = set(xml.xpath('//Layer/Layer/Name/text()'))
         expected_names = set(['direct', 'wms_cache', 'wms_cache_100', 'wms_cache_130',
-                              'wms_merge', 'tms_cache', 'wms_cache_multi',
-                              'wms_cache_link_single', 'wms_cache_110'])
+            'wms_cache_transparent', 'wms_merge', 'tms_cache', 'wms_cache_multi',
+            'wms_cache_link_single', 'wms_cache_110'])
         eq_(layer_names, expected_names)
         assert validate_with_dtd(xml, dtd_name='wms/1.1.0/capabilities_1_1_0.dtd')
 
@@ -519,14 +556,6 @@ class TestWMS110(WMSTest):
         assert is_png(data)
         assert Image.open(data).mode == 'RGB'
     
-    def test_get_map_png_transparent(self):
-        self.common_map_req.params['transparent'] = 'True'
-        resp = self.app.get(self.common_map_req)
-        eq_(resp.content_type, 'image/png')
-        data = StringIO(resp.body)
-        assert is_png(data)
-        assert Image.open(data).mode == 'RGBA'
-    
     def test_get_map_jpeg(self):
         self.common_map_req.params['format'] = 'image/jpeg'
         resp = self.app.get(self.common_map_req)
@@ -579,7 +608,6 @@ class TestWMS110(WMSTest):
                         {'body': 'info', 'headers': {'content-type': 'text/plain'}})
         with mock_httpd(('localhost', 42423), [expected_req]):
             resp = self.app.get(self.common_fi_req)
-            print resp.body
             eq_(resp.content_type, 'text/plain')
             eq_(resp.body, 'info')
             
@@ -587,7 +615,6 @@ class TestWMS110(WMSTest):
         self.common_fi_req.params['query_layers'] = 'tms_cache'
         self.common_fi_req.params['exceptions'] = 'application/vnd.ogc.se_xml'
         resp = self.app.get(self.common_fi_req)
-        print resp.body
         eq_(resp.content_type, 'application/vnd.ogc.se_xml')
         xml = resp.lxml
         eq_(xml.xpath('/ServiceExceptionReport/ServiceException/@code'), [])
@@ -614,11 +641,11 @@ class TestWMS100(WMSTest):
         eq_(resp.content_type, 'text/xml')
         xml = resp.lxml
         eq_(xml.xpath('/WMT_MS_Capabilities/Service/Title/text()')[0],
-            'MapProxy test fixture')
+            u'MapProxy test fixture \u2603')
         layer_names = set(xml.xpath('//Layer/Layer/Name/text()'))
         expected_names = set(['direct', 'wms_cache', 'wms_cache_100', 'wms_cache_130',
-                              'wms_merge', 'tms_cache', 'wms_cache_multi',
-                              'wms_cache_link_single', 'wms_cache_110'])
+            'wms_cache_transparent', 'wms_merge', 'tms_cache', 'wms_cache_multi',
+            'wms_cache_link_single', 'wms_cache_110'])
         eq_(layer_names, expected_names)
         #TODO srs
         assert validate_with_dtd(xml, dtd_name='wms/1.0.0/capabilities_1_0_0.dtd')
@@ -681,14 +708,6 @@ class TestWMS100(WMSTest):
         finally:
             base_config().image.paletted = False
             
-    def test_get_map_png_transparent(self):
-        self.common_map_req.params['transparent'] = 'True'
-        resp = self.app.get(self.common_map_req)
-        eq_(resp.content_type, 'image/png')
-        data = StringIO(resp.body)
-        assert is_png(data)
-        assert Image.open(data).mode == 'RGBA'
-    
     def test_get_map_jpeg(self):
         self.common_map_req.params['format'] = 'image/jpeg'
         resp = self.app.get(self.common_map_req)
@@ -729,7 +748,6 @@ class TestWMS100(WMSTest):
         self.common_fi_req.params['query_layers'] = 'tms_cache'
         self.common_fi_req.params['exceptions'] = 'application/vnd.ogc.se_xml'
         resp = self.app.get(self.common_fi_req)
-        print resp.body
         eq_(resp.content_type, 'text/xml')
         xml = resp.lxml
         assert 'tms_cache is not queryable' in xml.xpath('//WMTException/text()')[0]
@@ -764,12 +782,12 @@ class TestWMS130(WMSTest):
         eq_(resp.content_type, 'text/xml')
         xml = resp.lxml
         eq_xpath_wms130(xml, '/wms:WMS_Capabilities/wms:Service/wms:Title/text()',
-                        'MapProxy test fixture')
+            u'MapProxy test fixture \u2603')
         layer_names = set(xml.xpath('//wms:Layer/wms:Layer/wms:Name/text()',
                                     namespaces=ns130))
         expected_names = set(['direct', 'wms_cache', 'wms_cache_100', 'wms_cache_130',
-                              'wms_merge', 'tms_cache', 'wms_cache_multi',
-                              'wms_cache_link_single', 'wms_cache_110'])
+            'wms_cache_transparent', 'wms_merge', 'tms_cache', 'wms_cache_multi',
+            'wms_cache_link_single', 'wms_cache_110'])
         eq_(layer_names, expected_names)
         assert is_130_capa(xml)
     
@@ -828,14 +846,6 @@ class TestWMS130(WMSTest):
         assert is_png(data)
         assert Image.open(data).mode == 'RGB'
         
-    def test_get_map_png_transparent(self):
-        self.common_map_req.params['transparent'] = 'True'
-        resp = self.app.get(self.common_map_req)
-        eq_(resp.content_type, 'image/png')
-        data = StringIO(resp.body)
-        assert is_png(data)
-        assert Image.open(data).mode == 'RGBA'
-    
     def test_get_map_jpeg(self):
         self.common_map_req.params['format'] = 'image/jpeg'
         resp = self.app.get(self.common_map_req)
@@ -901,15 +911,15 @@ if sys.platform != 'win32':
     
         def test_get_map(self):
             link_name = 'wms_cache_link_single_EPSG900913/01/000/000/001/000/000/001.png'
-            real_name = 'wms_cache_link_single_EPSG900913/single_color_tiles/fe0005.png'
+            real_name = 'wms_cache_link_single_EPSG900913/single_color_tiles/fe00a0.png'
             self.created_tiles.append(link_name)
             self.created_tiles.append(real_name)
-            with tmp_image((256, 256), format='png', color='#fe0005') as img:
-                expected_req = ({'path': r'/service?LAYERs=foo,bar&SERVICE=WMS&FORMAT=image%2Fpng'
+            with tmp_image((256, 256), format='jpeg', color='#fe00a0') as img:
+                expected_req = ({'path': r'/service?LAYERs=foo,bar&SERVICE=WMS&FORMAT=image%2Fjpeg'
                                           '&REQUEST=GetMap&HEIGHT=256&SRS=EPSG%3A900913&styles='
                                           '&VERSION=1.1.1&BBOX=0.0,0.0,20037508.3428,20037508.3428'
                                           '&WIDTH=256'},
-                                {'body': img.read(), 'headers': {'content-type': 'image/png'}})
+                                {'body': img.read(), 'headers': {'content-type': 'image/jpeg'}})
                 with mock_httpd(('localhost', 42423), [expected_req]):
                     self.common_map_req.params['bbox'] = '0,0,180,90'
                     resp = self.app.get(self.common_map_req)
