@@ -1,36 +1,39 @@
 # This file is part of the MapProxy project.
 # Copyright (C) 2010 Omniscale <http://omniscale.de>
 # 
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU Affero General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
 # 
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU Affero General Public License for more details.
+#    http://www.apache.org/licenses/LICENSE-2.0
 # 
-# You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
 from __future__ import with_statement
 
+import os
 import time
 import sys
 
 from mapproxy.client.http import HTTPClient, HTTPClientError
 from mapproxy.client.tile import TMSClient, TileClient, TileURLTemplate
 from mapproxy.client.wms import WMSClient, WMSInfoClient
+from mapproxy.grid import tile_grid
 from mapproxy.layer import MapQuery, InfoQuery
 from mapproxy.request.wms import WMS111MapRequest, WMS100MapRequest,\
                                  WMS130MapRequest, WMS111FeatureInfoRequest
 from mapproxy.srs import SRS
+from mapproxy.test.unit.test_cache import MockHTTPClient
 from mapproxy.test.http import mock_httpd, query_eq, assert_query_eq
 from mapproxy.test.helper import assert_re, TempFile
 
 from nose.tools import eq_
 from nose.plugins.skip import SkipTest
+from nose.plugins.attrib import attr
 
 TESTSERVER_ADDRESS = ('127.0.0.1', 56413)
 TESTSERVER_URL = 'http://%s:%s' % TESTSERVER_ADDRESS
@@ -50,45 +53,46 @@ class TestHTTPClient(object):
                                                   {'status': '500', 'body': ''})]):
                 self.client.open(TESTSERVER_URL + '/')
         except HTTPClientError, e:
-            assert_re(e.args[0], r'HTTP Error \(.*\): 500')
+            assert_re(e.args[0], r'HTTP Error ".*": 500')
         else:
             assert False, 'expected HTTPClientError'
     def test_invalid_url_type(self):
         try:
             self.client.open('htp://example.org')
         except HTTPClientError, e:
-            assert_re(e.args[0], r'No response .* \(htp://example.*\): unknown url type')
+            assert_re(e.args[0], r'No response .* "htp://example.*": unknown url type')
         else:
             assert False, 'expected HTTPClientError'
     def test_invalid_url(self):
         try:
             self.client.open('this is not a url')
         except HTTPClientError, e:
-            assert_re(e.args[0], r'URL not correct \(this is not.*\): unknown url type')
+            assert_re(e.args[0], r'URL not correct "this is not.*": unknown url type')
         else:
             assert False, 'expected HTTPClientError'
     def test_unknown_host(self):
         try:
             self.client.open('http://thishostshouldnotexist000136really42.org')
         except HTTPClientError, e:
-            assert_re(e.args[0], r'No response .* \(http://thishost.*\): .*')
+            assert_re(e.args[0], r'No response .* "http://thishost.*": .*')
         else:
             assert False, 'expected HTTPClientError'
     def test_no_connect(self):
         try:
             self.client.open('http://localhost:53871')
         except HTTPClientError, e:
-            assert_re(e.args[0], r'No response .* \(http://localhost.*\): Connection refused')
+            assert_re(e.args[0], r'No response .* "http://localhost.*": Connection refused')
         else:
             assert False, 'expected HTTPClientError'
     
+    @attr('online')
     def test_https_no_ssl_module_error(self):
         from mapproxy.client import http
         old_ssl = http.ssl
         try:
             http.ssl = None
             try:
-                self.client = HTTPClient('https://trac.osgeo.org/')
+                self.client = HTTPClient('https://www.google.com/')
             except ImportError:
                 pass
             else:
@@ -96,28 +100,36 @@ class TestHTTPClient(object):
         finally:
             http.ssl = old_ssl
     
+    @attr('online')
     def test_https_no_ssl_module_insecure(self):
         from mapproxy.client import http
         old_ssl = http.ssl
         try:
             http.ssl = None
-            self.client = HTTPClient('https://trac.osgeo.org/', insecure=True)
-            self.client.open('https://trac.osgeo.org/')
+            self.client = HTTPClient('https://www.google.com/', insecure=True)
+            self.client.open('https://www.google.com/')
         finally:
             http.ssl = old_ssl
     
+    @attr('online')
     def test_https_valid_cert(self):
         try:
             import ssl; ssl
         except ImportError:
             raise SkipTest()
         
-        with TempFile() as tmp:
-            with open(tmp, 'w') as f:
-                f.write(OSGEO_CERT)
-            self.client = HTTPClient('https://trac.osgeo.org/', ssl_ca_certs=tmp)
-            self.client.open('https://trac.osgeo.org/')
+        cert_file = '/etc/ssl/certs/ca-certificates.crt'
+        if os.path.exists(cert_file):
+            self.client = HTTPClient('https://www.google.com/', ssl_ca_certs=cert_file)
+            self.client.open('https://www.google.com/')
+        else:
+            with TempFile() as tmp:
+                with open(tmp, 'w') as f:
+                    f.write(GOOGLE_ROOT_CERT)
+                self.client = HTTPClient('https://www.google.com/', ssl_ca_certs=tmp)
+                self.client.open('https://www.google.com/')
     
+    @attr('online')
     def test_https_invalid_cert(self):
         try:
             import ssl; ssl
@@ -125,9 +137,9 @@ class TestHTTPClient(object):
             raise SkipTest()
         
         with TempFile() as tmp:
-            self.client = HTTPClient('https://trac.osgeo.org/', ssl_ca_certs=tmp)
+            self.client = HTTPClient('https://www.google.com/', ssl_ca_certs=tmp)
             try:
-                self.client.open('https://trac.osgeo.org/')
+                self.client.open('https://www.google.com/')
             except HTTPClientError, e:
                 assert_re(e.args[0], r'Could not verify connection to URL')
         
@@ -173,37 +185,28 @@ class TestHTTPClient(object):
 
         mapproxy.client.http._max_set_timeout = old_timeout
 
-        
-OSGEO_CERT = """
+# Equifax Secure Certificate Authority
+# Expires: 2018-08-22
+GOOGLE_ROOT_CERT = """
 -----BEGIN CERTIFICATE-----
-MIIE2DCCBEGgAwIBAgIEN0rSQzANBgkqhkiG9w0BAQUFADCBwzELMAkGA1UEBhMC
-VVMxFDASBgNVBAoTC0VudHJ1c3QubmV0MTswOQYDVQQLEzJ3d3cuZW50cnVzdC5u
-ZXQvQ1BTIGluY29ycC4gYnkgcmVmLiAobGltaXRzIGxpYWIuKTElMCMGA1UECxMc
-KGMpIDE5OTkgRW50cnVzdC5uZXQgTGltaXRlZDE6MDgGA1UEAxMxRW50cnVzdC5u
-ZXQgU2VjdXJlIFNlcnZlciBDZXJ0aWZpY2F0aW9uIEF1dGhvcml0eTAeFw05OTA1
-MjUxNjA5NDBaFw0xOTA1MjUxNjM5NDBaMIHDMQswCQYDVQQGEwJVUzEUMBIGA1UE
-ChMLRW50cnVzdC5uZXQxOzA5BgNVBAsTMnd3dy5lbnRydXN0Lm5ldC9DUFMgaW5j
-b3JwLiBieSByZWYuIChsaW1pdHMgbGlhYi4pMSUwIwYDVQQLExwoYykgMTk5OSBF
-bnRydXN0Lm5ldCBMaW1pdGVkMTowOAYDVQQDEzFFbnRydXN0Lm5ldCBTZWN1cmUg
-U2VydmVyIENlcnRpZmljYXRpb24gQXV0aG9yaXR5MIGdMA0GCSqGSIb3DQEBAQUA
-A4GLADCBhwKBgQDNKIM0VBuJ8w+vN5Ex/68xYMmo6LIQaO2f55M28Qpku0f1BBc/
-I0dNxScZgSYMVHINiC3ZH5oSn7yzcdOAGT9HZnuMNSjSuQrfJNqc1lB5gXpa0zf3
-wkrYKZImZNHkmGw6AIr1NJtl+O3jEP/9uElY3KDegjlrgbEWGWG5VLbmQwIBA6OC
-AdcwggHTMBEGCWCGSAGG+EIBAQQEAwIABzCCARkGA1UdHwSCARAwggEMMIHeoIHb
-oIHYpIHVMIHSMQswCQYDVQQGEwJVUzEUMBIGA1UEChMLRW50cnVzdC5uZXQxOzA5
-BgNVBAsTMnd3dy5lbnRydXN0Lm5ldC9DUFMgaW5jb3JwLiBieSByZWYuIChsaW1p
-dHMgbGlhYi4pMSUwIwYDVQQLExwoYykgMTk5OSBFbnRydXN0Lm5ldCBMaW1pdGVk
-MTowOAYDVQQDEzFFbnRydXN0Lm5ldCBTZWN1cmUgU2VydmVyIENlcnRpZmljYXRp
-b24gQXV0aG9yaXR5MQ0wCwYDVQQDEwRDUkwxMCmgJ6AlhiNodHRwOi8vd3d3LmVu
-dHJ1c3QubmV0L0NSTC9uZXQxLmNybDArBgNVHRAEJDAigA8xOTk5MDUyNTE2MDk0
-MFqBDzIwMTkwNTI1MTYwOTQwWjALBgNVHQ8EBAMCAQYwHwYDVR0jBBgwFoAU8Bdi
-E1U9s/8KAGv7UISX8+1i0BowHQYDVR0OBBYEFPAXYhNVPbP/CgBr+1CEl/PtYtAa
-MAwGA1UdEwQFMAMBAf8wGQYJKoZIhvZ9B0EABAwwChsEVjQuMAMCBJAwDQYJKoZI
-hvcNAQEFBQADgYEAkNwwAvpkdMKnCqV8IY00F6j7Rw7/JXyNEwr75Ji174z4xRAN
-95K+8cPV1ZVqBLssziY2ZcgxxufuP+NXdYR6Ee9GTxj005i7qIcyunL2POI9n9cd
-2cNgQ4xYDiKWL2KjLB+6rQXvqzJ4h6BUcxm1XAX5Uj5tLUUL9wqT6u0G+bI=
+MIIDIDCCAomgAwIBAgIENd70zzANBgkqhkiG9w0BAQUFADBOMQswCQYDVQQGEwJV
+UzEQMA4GA1UEChMHRXF1aWZheDEtMCsGA1UECxMkRXF1aWZheCBTZWN1cmUgQ2Vy
+dGlmaWNhdGUgQXV0aG9yaXR5MB4XDTk4MDgyMjE2NDE1MVoXDTE4MDgyMjE2NDE1
+MVowTjELMAkGA1UEBhMCVVMxEDAOBgNVBAoTB0VxdWlmYXgxLTArBgNVBAsTJEVx
+dWlmYXggU2VjdXJlIENlcnRpZmljYXRlIEF1dGhvcml0eTCBnzANBgkqhkiG9w0B
+AQEFAAOBjQAwgYkCgYEAwV2xWGcIYu6gmi0fCG2RFGiYCh7+2gRvE4RiIcPRfM6f
+BeC4AfBONOziipUEZKzxa1NfBbPLZ4C/QgKO/t0BCezhABRP/PvwDN1Dulsr4R+A
+cJkVV5MW8Q+XarfCaCMczE1ZMKxRHjuvK9buY0V7xdlfUNLjUA86iOe/FP3gx7kC
+AwEAAaOCAQkwggEFMHAGA1UdHwRpMGcwZaBjoGGkXzBdMQswCQYDVQQGEwJVUzEQ
+MA4GA1UEChMHRXF1aWZheDEtMCsGA1UECxMkRXF1aWZheCBTZWN1cmUgQ2VydGlm
+aWNhdGUgQXV0aG9yaXR5MQ0wCwYDVQQDEwRDUkwxMBoGA1UdEAQTMBGBDzIwMTgw
+ODIyMTY0MTUxWjALBgNVHQ8EBAMCAQYwHwYDVR0jBBgwFoAUSOZo+SvSspXXR9gj
+IBBPM5iQn9QwHQYDVR0OBBYEFEjmaPkr0rKV10fYIyAQTzOYkJ/UMAwGA1UdEwQF
+MAMBAf8wGgYJKoZIhvZ9B0EABA0wCxsFVjMuMGMDAgbAMA0GCSqGSIb3DQEBBQUA
+A4GBAFjOKer89961zgK5F7WF0bnj4JXMJTENAKaSbn+2kmOeUJXRmm/kEd5jhW6Y
+7qj/WsjTVbJmcVfewCHrPSqnI0kBBIZCe/zuf6IWUrVnZ9NA2zsmWLIodz2uFHdh
+1voqZiegDfqnc1zqcPGUIWVEX/r87yloqaKHee9570+sB3c4
 -----END CERTIFICATE-----
-
 """
 
 # class TestWMSClient(object):
@@ -323,69 +326,24 @@ class TestTileClient(object):
             resp = client.get_tile((5, 13, 9)).source.read()
             eq_(resp, 'tile')
 
-from mapproxy.test.unit.test_cache import MockHTTPClient
-class TestWMSClient(object):
-    def setup(self):
-        self.req = WMS111MapRequest(url=TESTSERVER_URL + '/service?map=foo', param={'layers':'foo'})
-        self.http = MockHTTPClient()
-        self.wms = WMSClient(self.req, http_client=self.http, supported_srs=[SRS(4326)])
-    def test_request(self):
-        req = MapQuery((-180.0, -90.0, 180.0, 90.0), (512, 256), SRS(4326), 'png')
-        self.wms.get_map(req)
-        eq_(len(self.http.requested), 1)
-        assert_query_eq(self.http.requested[0],
-            TESTSERVER_URL+'/service?map=foo&LAYERS=foo&SERVICE=WMS&FORMAT=image%2Fpng'
-                           '&REQUEST=GetMap&HEIGHT=256&SRS=EPSG%3A4326'
-                           '&VERSION=1.1.1&BBOX=-180.0,-90.0,180.0,90.0&WIDTH=512&STYLES=')
+    def test_arcgiscache_path(self):
+        template = TileURLTemplate(TESTSERVER_URL + '/%(arcgiscache_path)s.png')
+        client = TileClient(template)
+        with mock_httpd(TESTSERVER_ADDRESS, [({'path': '/L09/R0000000d/C00000005.png'},
+                                              {'body': 'tile',
+                                               'headers': {'content-type': 'image/png'}})]):
+            resp = client.get_tile((5, 13, 9)).source.read()
+            eq_(resp, 'tile')
 
-    def test_transformed_request(self):
-        req = MapQuery((-200000, -200000, 200000, 200000), (512, 512), SRS(900913), 'png')
-        resp = self.wms.get_map(req)
-        eq_(len(self.http.requested), 1)
-        
-        assert_query_eq(self.http.requested[0], 
-            TESTSERVER_URL+'/service?map=foo&LAYERS=foo&SERVICE=WMS&FORMAT=image%2Fpng'
-                           '&REQUEST=GetMap&HEIGHT=512&SRS=EPSG%3A4326'
-                           '&VERSION=1.1.1&WIDTH=512&STYLES='
-                           '&BBOX=-1.79663056824,-1.7963362121,1.79663056824,1.7963362121')
-        img = resp.as_image()
-        assert img.mode in ('P', 'RGB')
-
-    def test_similar_srs(self):
-        # request in 3857 and source supports only 900913
-        # 3857 and 900913 are equal but the client requests must use 900913
-        self.req = WMS111MapRequest(url=TESTSERVER_URL + '/service?map=foo',
-                                    param={'layers':'foo', 'transparent': 'true'})
-        self.wms = WMSClient(self.req, http_client=self.http, supported_srs=[SRS(900913)])
-
-        req = MapQuery((-200000, -200000, 200000, 200000), (512, 512), SRS(3857), 'png')
-        self.wms.get_map(req)
-        eq_(len(self.http.requested), 1)
-        
-        assert_query_eq(self.http.requested[0],
-            TESTSERVER_URL+'/service?map=foo&LAYERS=foo&SERVICE=WMS&FORMAT=image%2Fpng'
-                           '&REQUEST=GetMap&HEIGHT=512&SRS=EPSG%3A900913'
-                           '&VERSION=1.1.1&WIDTH=512&STYLES=&transparent=true'
-                           '&BBOX=-200000,-200000,200000,200000')
-
-    def test_transformed_request_transparent(self):
-        self.req = WMS111MapRequest(url=TESTSERVER_URL + '/service?map=foo',
-                                    param={'layers':'foo', 'transparent': 'true'})
-        self.wms = WMSClient(self.req, http_client=self.http, supported_srs=[SRS(4326)])
-
-        req = MapQuery((-200000, -200000, 200000, 200000), (512, 512), SRS(900913), 'png')
-        resp = self.wms.get_map(req)
-        eq_(len(self.http.requested), 1)
-        
-        assert_query_eq(self.http.requested[0],
-            TESTSERVER_URL+'/service?map=foo&LAYERS=foo&SERVICE=WMS&FORMAT=image%2Fpng'
-                           '&REQUEST=GetMap&HEIGHT=512&SRS=EPSG%3A4326'
-                           '&VERSION=1.1.1&WIDTH=512&STYLES=&transparent=true'
-                           '&BBOX=-1.79663056824,-1.7963362121,1.79663056824,1.7963362121')
-        img = resp.as_image()
-        assert img.mode in ('P', 'RGBA')
-        img = img.convert('RGBA')
-        eq_(img.getpixel((5, 5))[3], 0)
+    def test_bbox(self):
+        grid = tile_grid(4326)
+        template = TileURLTemplate(TESTSERVER_URL + '/service?BBOX=%(bbox)s')
+        client = TileClient(template, grid=grid)
+        with mock_httpd(TESTSERVER_ADDRESS, [({'path': '/service?BBOX=-180.00000000,0.00000000,-90.00000000,90.00000000'},
+                                              {'body': 'tile',
+                                               'headers': {'content-type': 'image/png'}})]):
+            resp = client.get_tile((0, 1, 2)).source.read()
+            eq_(resp, 'tile')
 
 class TestCombinedWMSClient(object):
     def setup(self):
@@ -393,10 +351,10 @@ class TestCombinedWMSClient(object):
     def test_combine(self):
         req1 = WMS111MapRequest(url=TESTSERVER_URL + '/service?map=foo',
                                     param={'layers':'foo', 'transparent': 'true'})
-        wms1 = WMSClient(req1, http_client=self.http, supported_srs=[SRS(4326)])
+        wms1 = WMSClient(req1, http_client=self.http)
         req2 = WMS111MapRequest(url=TESTSERVER_URL + '/service?map=foo',
                                     param={'layers':'bar', 'transparent': 'true'})
-        wms2 = WMSClient(req2, http_client=self.http, supported_srs=[SRS(4326)])
+        wms2 = WMSClient(req2, http_client=self.http)
         
         req = MapQuery((-200000, -200000, 200000, 200000), (512, 512), SRS(900913), 'png')
         
@@ -407,10 +365,10 @@ class TestCombinedWMSClient(object):
     def test_combine_different_url(self):
         req1 = WMS111MapRequest(url=TESTSERVER_URL + '/service?map=bar',
                                     param={'layers':'foo', 'transparent': 'true'})
-        wms1 = WMSClient(req1, http_client=self.http, supported_srs=[SRS(4326)])
+        wms1 = WMSClient(req1, http_client=self.http)
         req2 = WMS111MapRequest(url=TESTSERVER_URL + '/service?map=foo',
                                     param={'layers':'bar', 'transparent': 'true'})
-        wms2 = WMSClient(req2, http_client=self.http, supported_srs=[SRS(4326)])
+        wms2 = WMSClient(req2, http_client=self.http)
         
         req = MapQuery((-200000, -200000, 200000, 200000), (512, 512), SRS(900913), 'png')
         
@@ -484,4 +442,4 @@ class TestWMSMapRequest111(object):
         self.r.params = self.r.adapt_params_to_version()
     def test_version(self):
         eq_(self.r.params['version'], '1.1.1')
-        assert 'WMTVER' not in self.r.params
\ No newline at end of file
+        assert 'WMTVER' not in self.r.params
