@@ -1,18 +1,17 @@
 # This file is part of the MapProxy project.
 # Copyright (C) 2010 Omniscale <http://omniscale.de>
 # 
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU Affero General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
 # 
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU Affero General Public License for more details.
+#    http://www.apache.org/licenses/LICENSE-2.0
 # 
-# You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
 from __future__ import division, with_statement
 from functools import partial
@@ -20,6 +19,9 @@ import operator
 
 from mapproxy.layer import MapExtent
 
+import logging
+log_config = logging.getLogger('mapproxy.config.coverage')
+
 try:
     import shapely.wkt
     import shapely.geometry
@@ -47,7 +49,15 @@ def load_datasource(datasource, where=None):
     
     polygons = []
     for wkt in OGRShapeReader(datasource).wkts(where):
-        polygons.append(shapely.wkt.loads(wkt))
+        geom = shapely.wkt.loads(wkt)
+        if geom.type == 'Polygon':
+            polygons.append(geom)
+        elif geom.type == 'MultiPolygon':
+            for p in geom:
+                polygons.append(p)
+        else:
+            log_config.warn('skipping %s geometry from %s: not a Polygon/MultiPolygon',
+                geom.type, datasource)
         
     mp = shapely.geometry.MultiPolygon(polygons)
     mp = simplify_geom(mp)
@@ -68,11 +78,14 @@ def load_polygons(geom_files):
         with open(geom_file) as f:
             for line in f:
                 geom = shapely.wkt.loads(line)
-                if geom.type != 'Polygon':
-                    print 'ignoring non-polygon geometry (%s) from %s' % \
-                        (geom.type, geom_file)
-                else:
+                if geom.type == 'Polygon':
                     polygons.append(geom)
+                elif geom.type == 'MultiPolygon':
+                    for p in geom:
+                        polygons.append(p)
+                else:
+                    log_config.warn('ignoring non-polygon geometry (%s) from %s',
+                        geom.type, geom_file)
     
     mp = shapely.geometry.MultiPolygon(polygons)
     mp = simplify_geom(mp)
